@@ -1,20 +1,18 @@
 ## -*- coding: utf-8 -*-
 ##
 ##  Jonathan Salwan - 2014-05-17 - ROPgadget tool
-## 
+##
 ##  http://twitter.com/JonathanSalwan
 ##  http://shell-storm.org/project/ROPgadget/
-## 
-##  This program is free software: you can redistribute it and/or modify
-##  it under the terms of the GNU General Public License as published by
-##  the Free Software  Foundation, either  version 3 of  the License, or
-##  (at your option) any later version.
+##
 
 import cmd
 import os
 import re
+import codecs
 import ropgadget.rgutils as rgutils
 import sqlite3
+import binascii
 
 from ropgadget.binary             import Binary
 from capstone                     import CS_MODE_32
@@ -33,13 +31,42 @@ class Core(cmd.Cmd):
 
 
     def __checksBeforeManipulations(self):
-        if self.__binary == None or self.__binary.getBinary() == None or self.__binary.getArch() == None or self.__binary.getArchMode() == None:
+        if self.__binary == None or self.__binary.getBinary() == None or self.__binary.getArch() == None or self.__binary.getArchMode() == None or self.__binary.getEndian() == None:
             return False
         return True
 
+    def _sectionInRange(self, section):
+        """
+        given a section and a range, edit the section so that all opcodes are within the range
+        """
+        if self.__options.range == "0x0-0x0":
+            return section
 
-    def __getAllgadgets(self):
+        rangeStart, rangeEnd = map(lambda x:int(x, 16), self.__options.range.split('-'))
 
+        sectionStart = section['vaddr']
+        sectionEnd = sectionStart + section['size']
+
+        opcodes = section['opcodes']
+        if rangeEnd < sectionStart or rangeStart > sectionEnd:
+            return None
+        if rangeStart > sectionStart:
+            diff = rangeStart - sectionStart
+            opcodes = opcodes[diff:]
+            section['vaddr'] += diff
+            section['offset'] += diff
+            section['size'] -= diff
+        if rangeEnd < sectionEnd:
+            diff = sectionEnd - rangeEnd
+            opcodes = opcodes[:-diff]
+            section['size'] -= diff
+
+        if not section['size']:
+            return None
+        section['opcodes'] = opcodes
+        return section
+
+    def __getGadgets(self):
         if self.__checksBeforeManipulations() == False:
             return False
 
@@ -49,22 +76,22 @@ class Core(cmd.Cmd):
         # Find ROP/JOP/SYS gadgets
         self.__gadgets = []
         for section in execSections:
+            section = self._sectionInRange(section)
+            if not section: continue
             if not self.__options.norop: self.__gadgets += G.addROPGadgets(section)
             if not self.__options.nojop: self.__gadgets += G.addJOPGadgets(section)
             if not self.__options.nosys: self.__gadgets += G.addSYSGadgets(section)
 
-        # Pass clean single instruction and unknown instructions
-        self.__gadgets = G.passClean(self.__gadgets, self.__options.multibr)
-
         # Delete duplicate gadgets
-        if not self.__options.all:
+        if not self.__options.all and not self.__options.noinstr:
             self.__gadgets = rgutils.deleteDuplicateGadgets(self.__gadgets)
 
         # Applicate some Options
         self.__gadgets = Options(self.__options, self.__binary, self.__gadgets).getGadgets()
 
         # Sorted alphabetically
-        self.__gadgets = rgutils.alphaSortgadgets(self.__gadgets)
+        if not self.__options.noinstr:
+            self.__gadgets = rgutils.alphaSortgadgets(self.__gadgets)
 
         return True
 
@@ -74,12 +101,19 @@ class Core(cmd.Cmd):
         if self.__checksBeforeManipulations() == False:
             return False
 
+        if self.__options.silent:
+            return True
+
         arch = self.__binary.getArchMode()
         print("Gadgets information\n============================================================")
         for gadget in self.__gadgets:
             vaddr = gadget["vaddr"]
-            insts = gadget["gadget"]
-            print(("0x%08x" %(vaddr) if arch == CS_MODE_32 else "0x%016x" %(vaddr)) + " : %s" %(insts))
+            insts = gadget.get("gadget", "")
+            bytesStr = " // " + binascii.hexlify(gadget["bytes"]).decode('utf8') if self.__options.dump else ""
+
+            print(("0x%08x" %(vaddr) if arch == CS_MODE_32 else "0x%016x" %(vaddr)) +
+                  (" : %s" %(insts) if insts else "") + bytesStr)
+
         print("\nUnique gadgets found: %d" %(len(self.__gadgets)))
         return True
 
@@ -89,45 +123,52 @@ class Core(cmd.Cmd):
         if self.__checksBeforeManipulations() == False:
             return False
 
+        if self.__options.silent:
+            return True
+
         dataSections = self.__binary.getDataSections()
         arch = self.__binary.getArchMode()
         print("Strings information\n============================================================")
         for section in dataSections:
-            allRef = [m.start() for m in re.finditer(string, section["opcodes"])]
+            section = self._sectionInRange(section)
+            if not section: continue
+            allRef = [m.start() for m in re.finditer(string.encode(), section["opcodes"])]
             for ref in allRef:
                 vaddr  = self.__offset + section["vaddr"] + ref
-                string = section["opcodes"][ref:ref+len(string)]
-                rangeS = int(self.__options.range.split('-')[0], 16)
-                rangeE = int(self.__options.range.split('-')[1], 16)
-                if (rangeS == 0 and rangeE == 0) or (vaddr >= rangeS and vaddr <= rangeE):
-                    print(("0x%08x" %(vaddr) if arch == CS_MODE_32 else "0x%016x" %(vaddr)) + " : %s" %(string))
+                match = section["opcodes"][ref:ref+len(string)]
+                print(("0x%08x" %(vaddr) if arch == CS_MODE_32 else "0x%016x" %(vaddr)) + " : %s" %(match.decode()))
         return True
 
 
     def __lookingForOpcodes(self, opcodes):
+        import binascii
 
         if self.__checksBeforeManipulations() == False:
             return False
 
+        if self.__options.silent:
+            return True
+
         execSections = self.__binary.getExecSections()
         arch = self.__binary.getArchMode()
         print("Opcodes information\n============================================================")
         for section in execSections:
-            allRef = [m.start() for m in re.finditer(opcodes.decode("hex"), section["opcodes"])]
+            section = self._sectionInRange(section)
+            if not section: continue
+            allRef = [m.start() for m in re.finditer(re.escape(binascii.unhexlify(opcodes)), section["opcodes"])]
             for ref in allRef:
                 vaddr  = self.__offset + section["vaddr"] + ref
-                rangeS = int(self.__options.range.split('-')[0], 16)
-                rangeE = int(self.__options.range.split('-')[1], 16)
-                if (rangeS == 0 and rangeE == 0) or (vaddr >= rangeS and vaddr <= rangeE):
-                    print(("0x%08x" %(vaddr) if arch == CS_MODE_32 else "0x%016x" %(vaddr)) + " : %s" %(opcodes))
+                print(("0x%08x" %(vaddr) if arch == CS_MODE_32 else "0x%016x" %(vaddr)) + " : %s" %(opcodes))
         return True
 
 
     def __lookingForMemStr(self, memstr):
-
         if self.__checksBeforeManipulations() == False:
             return False
 
+        if self.__options.silent:
+            return True
+
         sections  = self.__binary.getExecSections()
         sections += self.__binary.getDataSections()
         arch = self.__binary.getArchMode()
@@ -136,14 +177,13 @@ class Core(cmd.Cmd):
         for char in chars:
             try:
                 for section in sections:
-                    allRef = [m.start() for m in re.finditer(char, section["opcodes"])]
+                    section = self._sectionInRange(section)
+                    if not section: continue
+                    allRef = [m.start() for m in re.finditer(char.encode('utf-8'), section["opcodes"])]
                     for ref in allRef:
                         vaddr  = self.__offset + section["vaddr"] + ref
-                        rangeS = int(self.__options.range.split('-')[0], 16)
-                        rangeE = int(self.__options.range.split('-')[1], 16)
-                        if (rangeS == 0 and rangeE == 0) or (vaddr >= rangeS and vaddr <= rangeE):
-                            print(("0x%08x" %(vaddr) if arch == CS_MODE_32 else "0x%016x" %(vaddr)) + " : '%c'" %(char))
-                            raise
+                        print(("0x%08x" %(vaddr) if arch == CS_MODE_32 else "0x%016x" %(vaddr)) + " : '%c'" %(char))
+                        raise
             except:
                 pass
         return True
@@ -172,8 +212,8 @@ class Core(cmd.Cmd):
         if   self.__options.string:   return self.__lookingForAString(self.__options.string)
         elif self.__options.opcode:   return self.__lookingForOpcodes(self.__options.opcode)
         elif self.__options.memstr:   return self.__lookingForMemStr(self.__options.memstr)
-        else: 
-            self.__getAllgadgets()
+        else:
+            self.__getGadgets()
             self.__lookingForGadgets()
             if self.__options.ropchain:
                 ROPMaker(self.__binary, self.__gadgets, self.__offset)
@@ -189,7 +229,7 @@ class Core(cmd.Cmd):
     # Console methods  ============================================
 
     def do_binary(self, s, silent=False):
-        # Do not split the filename with spaces since it might contain 
+        # Do not split the filename with spaces since it might contain
         # whitespaces
         if len(s) == 0:
             if not silent:
@@ -233,12 +273,12 @@ class Core(cmd.Cmd):
 
         if not silent:
             print("[+] Loading gadgets, please wait...")
-        self.__getAllgadgets()
+        self.__getGadgets()
 
         if not silent:
             print("[+] Gadgets loaded !")
 
-        
+
     def help_load(self):
         print("Syntax: load -- Load all gadgets")
         return False
@@ -301,7 +341,7 @@ class Core(cmd.Cmd):
             if a not in gadget:
                 return False
         return True
-        
+
     def __withoutK(self, listK, gadget):
         for a in listK:
             if a in gadget:
@@ -334,7 +374,7 @@ class Core(cmd.Cmd):
     def help_search(self):
         print("Syntax: search <keyword1 keyword2 keyword3...> -- Filter with or without keywords")
         print("keyword  = with")
-        print("!keyword = witout")
+        print("!keyword = without")
         return False
 
 
@@ -364,13 +404,16 @@ class Core(cmd.Cmd):
 
 
     def help_filter(self):
-        print("Syntax: filter <filter1|filter2|...> - Suppress specific instructions")
+        print("Syntax: filter <filter1|filter2|...> - Suppress specific mnemonics")
         return False
 
 
     def do_only(self, s, silent=False):
         try:
-            self.__options.only = s.split()[0]
+            if s.lower() == "none":
+                self.__options.only = None
+            else:
+                self.__options.only = s.split()[0]
         except:
             if not silent:
                 return self.help_only()
@@ -427,6 +470,8 @@ class Core(cmd.Cmd):
         print("Range:       %s" %(self.__options.range))
         print("RawArch:     %s" %(self.__options.rawArch))
         print("RawMode:     %s" %(self.__options.rawMode))
+        print("RawEndian:   %s" %(self.__options.rawEndian))
+        print("Re:          %s" %(self.__options.re))
         print("String:      %s" %(self.__options.string))
         print("Thumb:       %s" %(self.__options.thumb))
 
@@ -586,6 +631,23 @@ class Core(cmd.Cmd):
 
 
     def help_all(self):
-        print("Syntax: all <enable|disable - Show all gadgets (disable removing duplice gadgets)")
+        print("Syntax: all <enable|disable - Show all gadgets (disable removing duplicate gadgets)")
         return False
 
+
+    def help_re(self):
+        print("Syntax: re <pattern1 | pattern2 |...> - Regular expression")
+        return False
+
+
+    def do_re(self, s, silent=False):
+        if s.lower() == 'none':
+            self.__options.re = None
+        elif s == "":
+            self.help_re()
+            silent = True
+        else:
+            self.__options.re = s
+
+        if not silent:
+            print("[+] Re setted. You have to reload gadgets")
